@@ -2,7 +2,7 @@ import os
 import localstack_client.config
 
 # LocalStack version
-VERSION = '0.10.5'
+VERSION = '0.10.6'
 
 # constant to represent the "local" region, i.e., local machine
 REGION_LOCAL = 'local'
@@ -22,13 +22,15 @@ DEFAULT_PORT_CLOUDFORMATION_BACKEND = 4559
 DEFAULT_PORT_STEPFUNCTIONS_BACKEND = 4558
 DEFAULT_PORT_IAM_BACKEND = 4557
 DEFAULT_PORT_EC2_BACKEND = 4556
+DEFAULT_PORT_KMS_BACKEND = 4555
+DEFAULT_PORT_EVENTS_BACKEND = 4554
 
 DEFAULT_PORT_WEB_UI = 8080
 
 LOCALHOST = 'localhost'
 
 # version of the Maven dependency with Java utility code
-LOCALSTACK_MAVEN_VERSION = '0.1.22'
+LOCALSTACK_MAVEN_VERSION = '0.2.0'
 
 # map of default service APIs and ports to be spun up (fetch map from localstack_client)
 DEFAULT_SERVICE_PORTS = localstack_client.config.get_service_ports()
@@ -85,6 +87,7 @@ ELASTICSEARCH_DELETE_MODULES = ['ingest-geoip']
 ELASTICMQ_JAR_URL = 'https://s3-eu-west-1.amazonaws.com/softwaremill-public/elasticmq-server-0.15.2.jar'
 STS_JAR_URL = 'https://repo1.maven.org/maven2/com/amazonaws/aws-java-sdk-sts/1.11.14/aws-java-sdk-sts-1.11.14.jar'
 STEPFUNCTIONS_ZIP_URL = 'https://s3.amazonaws.com/stepfunctionslocal/StepFunctionsLocal.zip'
+KMS_URL_PATTERN = 'https://github.com/localstack/localstack-artifacts/raw/master/local-kms/build/local-kms.<arch>.bin'
 
 # TODO: Temporarily using a fixed version of DDB in Alpine, as we're hitting a SIGSEGV JVM crash with latest
 DYNAMODB_JAR_URL_ALPINE = 'https://github.com/whummer/dynamodb-local/raw/master/etc/DynamoDBLocal.zip'
