@@ -18,7 +18,7 @@ from localstack.services.generic_proxy import ProxyListener
 from localstack.utils.aws.aws_responses import flask_to_requests_response, requests_response
 from localstack.services.apigateway.helpers import (get_resource_for_path,
     handle_authorizers, extract_query_string_params,
-    extract_path_params, make_error, get_cors_response)
+    extract_path_params, make_error_response, get_cors_response)
 
 # set up logger
 LOGGER = logging.getLogger(__name__)
@@ -32,17 +32,25 @@ PATH_REGEX_USER_REQUEST = r'^/restapis/([A-Za-z0-9_\-]+)/([A-Za-z0-9_\-]+)/%s/(.
 GATEWAY_RESPONSES = {}
 
 
+class AuthorizationError(Exception):
+    pass
+
+
 class ProxyListenerApiGateway(ProxyListener):
 
     def forward_request(self, method, path, data, headers):
-        data = data and json.loads(to_str(data))
 
         if re.match(PATH_REGEX_USER_REQUEST, path):
             search_match = re.search(PATH_REGEX_USER_REQUEST, path)
             api_id = search_match.group(1)
             stage = search_match.group(2)
             relative_path_w_query_params = '/%s' % search_match.group(3)
-            return invoke_rest_api(api_id, stage, method, relative_path_w_query_params, data, headers, path=path)
+            try:
+                return invoke_rest_api(api_id, stage, method, relative_path_w_query_params, data, headers, path=path)
+            except AuthorizationError as e:
+                return make_error_response('Not authorized to invoke REST API %s: %s' % (api_id, e), 403)
+
+        data = data and json.loads(to_str(data))
 
         if re.match(PATH_REGEX_AUTHORIZERS, path):
             return handle_authorizers(method, path, data, headers)
@@ -134,15 +142,30 @@ def put_gateway_response(api_id, response_type, data):
     return data
 
 
+def run_authorizer(api_id, headers, authorizer):
+    # TODO implement authorizers
+    pass
+
+
+def authorize_invocation(api_id, headers):
+    client = aws_stack.connect_to_service('apigateway')
+    authorizers = client.get_authorizers(restApiId=api_id, limit=100).get('items', [])
+    for authorizer in authorizers:
+        run_authorizer(api_id, headers, authorizer)
+
+
 def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=None):
     path = path or invocation_path
     relative_path, query_string_params = extract_query_string_params(path=invocation_path)
 
+    # run gateway authorizers for this request
+    authorize_invocation(api_id, headers)
+
     path_map = helpers.get_rest_api_paths(rest_api_id=api_id)
     try:
         extracted_path, resource = get_resource_for_path(path=relative_path, path_map=path_map)
     except Exception:
-        return make_error('Unable to find path %s' % path, 404)
+        return make_error_response('Unable to find path %s' % path, 404)
 
     integrations = resource.get('resourceMethods', {})
     integration = integrations.get(method, {})
@@ -153,37 +176,41 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
         if method == 'OPTIONS' and 'Origin' in headers:
             # default to returning CORS headers if this is an OPTIONS request
             return get_cors_response(headers)
-        return make_error('Unable to find integration for path %s' % path, 404)
+        return make_error_response('Unable to find integration for path %s' % path, 404)
 
     uri = integration.get('uri')
-    if method == 'POST' and integration['type'] == 'AWS':
-        if uri.endswith('kinesis:action/PutRecords'):
+    if integration['type'] == 'AWS':
+        if 'kinesis:action/' in uri:
+            if uri.endswith('kinesis:action/PutRecords'):
+                target = kinesis_listener.ACTION_PUT_RECORDS
+            if uri.endswith('kinesis:action/ListStreams'):
+                target = kinesis_listener.ACTION_LIST_STREAMS
+
             template = integration['requestTemplates'][APPLICATION_JSON]
             new_request = aws_stack.render_velocity_template(template, data)
-
             # forward records to target kinesis stream
             headers = aws_stack.mock_aws_request_headers(service='kinesis')
-            headers['X-Amz-Target'] = kinesis_listener.ACTION_PUT_RECORDS
+            headers['X-Amz-Target'] = target
             result = common.make_http_request(url=TEST_KINESIS_URL,
                 method='POST', data=new_request, headers=headers)
             return result
 
-        elif uri.startswith('arn:aws:apigateway:') and ':sqs:path' in uri:
-            template = integration['requestTemplates'][APPLICATION_JSON]
-            account_id, queue = uri.split('/')[-2:]
-            region_name = uri.split(':')[3]
+        if method == 'POST':
+            if uri.startswith('arn:aws:apigateway:') and ':sqs:path' in uri:
+                template = integration['requestTemplates'][APPLICATION_JSON]
+                account_id, queue = uri.split('/')[-2:]
+                region_name = uri.split(':')[3]
 
-            new_request = aws_stack.render_velocity_template(template, data) + '&QueueName=%s' % queue
-            headers = aws_stack.mock_aws_request_headers(service='sqs', region_name=region_name)
+                new_request = aws_stack.render_velocity_template(template, data) + '&QueueName=%s' % queue
+                headers = aws_stack.mock_aws_request_headers(service='sqs', region_name=region_name)
 
-            url = urljoin(TEST_SQS_URL, '%s/%s' % (account_id, queue))
-            result = common.make_http_request(url, method='POST', headers=headers, data=new_request)
-            return result
+                url = urljoin(TEST_SQS_URL, '%s/%s' % (account_id, queue))
+                result = common.make_http_request(url, method='POST', headers=headers, data=new_request)
+                return result
 
-        else:
-            msg = 'API Gateway action uri "%s" not yet implemented' % uri
-            LOGGER.warning(msg)
-            return make_error(msg, 404)
+        msg = 'API Gateway AWS integration action URI "%s", method "%s" not yet implemented' % (uri, method)
+        LOGGER.warning(msg)
+        return make_error_response(msg, 404)
 
     elif integration['type'] == 'AWS_PROXY':
         if uri.startswith('arn:aws:apigateway:') and ':lambda:path' in uri:
@@ -239,7 +266,7 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
         else:
             msg = 'API Gateway action uri "%s" not yet implemented' % uri
             LOGGER.warning(msg)
-            return make_error(msg, 404)
+            return make_error_response(msg, 404)
 
     elif integration['type'] == 'HTTP':
         function = getattr(requests, method.lower())
@@ -252,7 +279,7 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
         msg = ('API Gateway integration type "%s" for method "%s" not yet implemented' %
                (integration['type'], method))
         LOGGER.warning(msg)
-        return make_error(msg, 404)
+        return make_error_response(msg, 404)
 
     return 200
 
