@@ -68,8 +68,9 @@ def set_moto_account_ids(resource_json):
     def fix_ids(obj, **kwargs):
         if isinstance(obj, dict):
             for key, value in obj.items():
-                if 'arn' in key.lower() and isinstance(value, six.string_types):
-                    obj[key] = value.replace(TEST_AWS_ACCOUNT_ID, MOTO_ACCOUNT_ID)
+                if isinstance(value, six.string_types):
+                    if 'arn' in key.lower() or (':%s:' % TEST_AWS_ACCOUNT_ID) in value:
+                        obj[key] = value.replace(TEST_AWS_ACCOUNT_ID, MOTO_ACCOUNT_ID)
         return obj
 
     return recurse_object(resource_json, fix_ids)
@@ -608,6 +609,19 @@ def apply_patches():
 
     responses.CloudFormationResponse.describe_stack_resource = describe_stack_resource
 
+    # fix moto's describe_stack_events jinja2.exceptions.UndefinedError
+
+    def cf_describe_stack_events(self):
+        stack_name = self._get_param('StackName')
+        stack = self.cloudformation_backend.get_stack(stack_name)
+        if not stack:
+            raise ValidationError(stack_name,
+                message='Unable to find stack "%s" in region %s' % (stack_name, aws_stack.get_region()))
+        return cf_describe_stack_events_orig(self)
+
+    cf_describe_stack_events_orig = responses.CloudFormationResponse.describe_stack_events
+    responses.CloudFormationResponse.describe_stack_events = cf_describe_stack_events
+
 
 def inject_stats_endpoint():
     """ Inject a simple /_stats endpoint into the moto server backend Web app. """
