@@ -14,6 +14,7 @@ API_PREFIX = '/2015-01-01'
 ES_DOMAINS = {}
 
 app = Flask(APP_NAME)
+app.url_map.strict_slashes = False
 
 
 def error_response(error_type, code=400, message='Unknown error.'):
@@ -73,6 +74,9 @@ def get_domain_config(domain_name):
                 },
                 'Status': config_status
             },
+            'CognitoOptions': {
+                'Enabled': False
+            },
             'ElasticsearchVersion': {
                 'Options': '5.3',
                 'Status': config_status
@@ -147,6 +151,9 @@ def get_domain_status(domain_name, deleted=False):
                 'VolumeSize': 10,
                 'Iops': 0
             },
+            'CognitoOptions': {
+                'Enabled': False
+            },
         }
     }
 
