@@ -13,7 +13,7 @@ import dateutil.parser
 from six.moves.urllib import parse as urlparse
 from botocore.client import ClientError
 from requests.models import Response, Request
-from localstack import config
+from localstack import config, constants
 from localstack.config import HOSTNAME, HOSTNAME_EXTERNAL
 from localstack.utils import persistence
 from localstack.utils.aws import aws_stack
@@ -30,18 +30,34 @@ S3_NOTIFICATIONS = {}
 # mappings for bucket CORS settings
 BUCKET_CORS = {}
 
-# mappings for bucket lifecycle settings
+# maps bucket name to lifecycle settings
 BUCKET_LIFECYCLE = {}
 
+# maps bucket name to replication settings
+BUCKET_REPLICATIONS = {}
+
+# maps bucket name to encryption settings
+BUCKET_ENCRYPTIONS = {}
+
+# maps bucket name to object lock settings
+OBJECT_LOCK_CONFIGS = {}
+
 # set up logger
 LOGGER = logging.getLogger(__name__)
 
 # XML namespace constants
 XMLNS_S3 = 'http://s3.amazonaws.com/doc/2006-03-01/'
 
+# see https://stackoverflow.com/questions/50480924/regex-for-s3-bucket-name#50484916
+BUCKET_NAME_REGEX = (r'(?=^.{3,63}$)(?!^(\d+\.)+\d+$)' +
+    r'(^(([a-z0-9]|[a-z0-9][a-z0-9\-]*[a-z0-9])\.)*([a-z0-9]|[a-z0-9][a-z0-9\-]*[a-z0-9])$)')
+
 # list of destination types for bucket notifications
 NOTIFICATION_DESTINATION_TYPES = ('Queue', 'Topic', 'CloudFunction', 'LambdaFunction')
 
+# prefix for object metadata keys in headers and query params
+OBJECT_METADATA_KEY_PREFIX = 'x-amz-meta-'
+
 # response header overrides the client may request
 ALLOWED_HEADER_OVERRIDES = {
     'response-content-type': 'Content-Type',
@@ -71,10 +87,11 @@ def filter_rules_match(filters, object_path):
     filters = filters or {}
     s3_filter = _get_s3_filter(filters)
     for rule in s3_filter.get('FilterRule', []):
-        if rule['Name'] == 'prefix':
+        rule_name_lower = rule['Name'].lower()
+        if rule_name_lower == 'prefix':
             if not prefix_with_slash(object_path).startswith(prefix_with_slash(rule['Value'])):
                 return False
-        elif rule['Name'] == 'suffix':
+        elif rule_name_lower == 'suffix':
             if not object_path.endswith(rule['Value']):
                 return False
         else:
@@ -92,6 +109,7 @@ def prefix_with_slash(s):
 
 def get_event_message(event_name, bucket_name, file_name='testfile.txt', version_id=None, file_size=1024):
     # Based on: http://docs.aws.amazon.com/AmazonS3/latest/dev/notification-content-structure.html
+    bucket_name = normalize_bucket_name(bucket_name)
     return {
         'Records': [{
             'eventVersion': '2.0',
@@ -132,6 +150,8 @@ def get_event_message(event_name, bucket_name, file_name='testfile.txt', version
 
 
 def queue_url_for_arn(queue_arn):
+    if '://' in queue_arn:
+        return queue_arn
     sqs_client = aws_stack.connect_to_service('sqs')
     parts = queue_arn.split(':')
     return sqs_client.get_queue_url(QueueName=parts[5],
@@ -139,6 +159,7 @@ def queue_url_for_arn(queue_arn):
 
 
 def send_notifications(method, bucket_name, object_path, version_id):
+    bucket_name = normalize_bucket_name(bucket_name)
     for bucket, notifs in S3_NOTIFICATIONS.items():
         if bucket == bucket_name:
             action = {'PUT': 'ObjectCreated', 'POST': 'ObjectCreated', 'DELETE': 'ObjectRemoved'}[method]
@@ -156,6 +177,8 @@ def send_notifications(method, bucket_name, object_path, version_id):
 
 
 def send_notification_for_subscriber(notif, bucket_name, object_path, version_id, api_method, action, event_name):
+    bucket_name = normalize_bucket_name(bucket_name)
+
     if (not event_type_matches(notif['Event'], action, api_method) or
             not filter_rules_match(notif.get('Filter'), object_path)):
         return
@@ -199,6 +222,7 @@ def send_notification_for_subscriber(notif, bucket_name, object_path, version_id
 
 
 def get_cors(bucket_name):
+    bucket_name = normalize_bucket_name(bucket_name)
     response = Response()
 
     exists, code = bucket_exists(bucket_name)
@@ -218,6 +242,7 @@ def get_cors(bucket_name):
 
 
 def set_cors(bucket_name, cors):
+    bucket_name = normalize_bucket_name(bucket_name)
     response = Response()
 
     exists, code = bucket_exists(bucket_name)
@@ -233,6 +258,7 @@ def set_cors(bucket_name, cors):
 
 
 def delete_cors(bucket_name):
+    bucket_name = normalize_bucket_name(bucket_name)
     response = Response()
 
     exists, code = bucket_exists(bucket_name)
@@ -246,6 +272,8 @@ def delete_cors(bucket_name):
 
 
 def append_cors_headers(bucket_name, request_method, request_headers, response):
+    bucket_name = normalize_bucket_name(bucket_name)
+
     cors = BUCKET_CORS.get(bucket_name)
     if not cors:
         return
@@ -295,54 +323,155 @@ def append_last_modified_headers(response, content=None):
         LOGGER.error('Caught generic exception (setting LastModified header): %s' % err)
 
 
+def append_list_objects_marker(method, path, data, response):
+    if 'marker=' in path:
+        content = to_str(response.content)
+        if '<ListBucketResult' in content and '<Marker>' not in content:
+            parsed = urlparse.urlparse(path)
+            query_map = urlparse.parse_qs(parsed.query)
+            insert = '<Marker>%s</Marker>' % query_map.get('marker')[0]
+            response._content = content.replace('</ListBucketResult>', '%s</ListBucketResult>' % insert)
+            response.headers['Content-Length'] = str(len(response._content))
+
+
+def append_metadata_headers(method, query_map, headers):
+    for key, value in query_map.items():
+        if key.lower().startswith(OBJECT_METADATA_KEY_PREFIX):
+            if headers.get(key) is None:
+                headers[key] = value[0]
+
+
+def fix_location_constraint(response):
+    """ Make sure we return a valid non-empty LocationConstraint, as this otherwise breaks Serverless. """
+    try:
+        content = to_str(response.content or '') or ''
+    except Exception:
+        content = ''
+    if 'LocationConstraint' in content:
+        pattern = r'<LocationConstraint([^>]*)>\s*</LocationConstraint>'
+        replace = r'<LocationConstraint\1>%s</LocationConstraint>' % aws_stack.get_region()
+        response._content = re.sub(pattern, replace, content)
+        remove_xml_preamble(response)
+
+
+def remove_xml_preamble(response):
+    """ Removes <?xml ... ?> from a response content """
+    response._content = re.sub(r'^<\?[^\?]+\?>', '', response._content)
+
+
+# --------------
+# HELPER METHODS
+#   for lifecycle/replication/encryption/...
+# --------------
+
+
 def get_lifecycle(bucket_name):
+    bucket_name = normalize_bucket_name(bucket_name)
     lifecycle = BUCKET_LIFECYCLE.get(bucket_name)
+    status_code = 200
     if not lifecycle:
-        # TODO: check if bucket exists, otherwise return 404-like error
+        # TODO: check if bucket actually exists
         lifecycle = {
-            'LifecycleConfiguration': {}
+            'Error': {
+                'Code': 'NoSuchLifecycleConfiguration',
+                'Message': 'The lifecycle configuration does not exist'
+            }
         }
+        status_code = 404
     body = xmltodict.unparse(lifecycle)
-    return requests_response(body)
+    return requests_response(body, status_code=status_code)
 
 
 def get_replication(bucket_name):
-    # TODO return actual value
-
-    # result = {
-    #     'Error': {
-    #         'Code': 'NoSuchReplicationConfiguration',
-    #         'Message': 'There is no replication configuration with that name.'
-    #     }
-    # }
-    # content = xmltodict.unparse(result)
-    # return requests_response(content, status_code=404)
-    # see https://docs.aws.amazon.com/AmazonS3/latest/API/RESTBucketGETreplication.html
-    config = {}
-    result = {
-        'ReplicationConfiguration': config
-    }
-    body = xmltodict.unparse(result)
-    return requests_response(body)
+    bucket_name = normalize_bucket_name(bucket_name)
+    replication = BUCKET_REPLICATIONS.get(bucket_name)
+    status_code = 200
+    if not replication:
+        # TODO: check if bucket actually exists
+        replication = {
+            'Error': {
+                'Code': 'ReplicationConfigurationNotFoundError',
+                'Message': 'The replication configuration was not found'
+            }
+        }
+        status_code = 404
+    body = xmltodict.unparse(replication)
+    return requests_response(body, status_code=status_code)
 
 
 def get_encryption(bucket_name):
-    # TODO return actual value
-    result = {
-        'ServerSideEncryptionConfiguration': {}
-    }
-    body = xmltodict.unparse(result)
-    return requests_response(body)
+    bucket_name = normalize_bucket_name(bucket_name)
+    encryption = BUCKET_ENCRYPTIONS.get(bucket_name)
+    status_code = 200
+    if not encryption:
+        # TODO: check if bucket actually exists
+        encryption = {
+            'Error': {
+                'Code': 'ServerSideEncryptionConfigurationNotFoundError',
+                'Message': 'The server side encryption configuration was not found'
+            }
+        }
+        status_code = 404
+    body = xmltodict.unparse(encryption)
+    return requests_response(body, status_code=status_code)
+
+
+def get_object_lock(bucket_name):
+    bucket_name = normalize_bucket_name(bucket_name)
+    lock_config = OBJECT_LOCK_CONFIGS.get(bucket_name)
+    status_code = 200
+    if not lock_config:
+        # TODO: check if bucket actually exists
+        lock_config = {
+            'Error': {
+                'Code': 'ObjectLockConfigurationNotFoundError',
+                'Message': 'Object Lock configuration does not exist for this bucket'
+            }
+        }
+        status_code = 404
+    body = xmltodict.unparse(lock_config)
+    return requests_response(body, status_code=status_code)
 
 
 def set_lifecycle(bucket_name, lifecycle):
+    bucket_name = normalize_bucket_name(bucket_name)
     # TODO: check if bucket exists, otherwise return 404-like error
     if isinstance(to_str(lifecycle), six.string_types):
         lifecycle = xmltodict.parse(lifecycle)
     BUCKET_LIFECYCLE[bucket_name] = lifecycle
-    response = Response()
-    response.status_code = 200
-    return response
+    return 200
+
+
+def set_replication(bucket_name, replication):
+    bucket_name = normalize_bucket_name(bucket_name)
+    # TODO: check if bucket exists, otherwise return 404-like error
+    if isinstance(to_str(replication), six.string_types):
+        replication = xmltodict.parse(replication)
+    BUCKET_REPLICATIONS[bucket_name] = replication
+    return 200
+
+
+def set_encryption(bucket_name, encryption):
+    bucket_name = normalize_bucket_name(bucket_name)
+    # TODO: check if bucket exists, otherwise return 404-like error
+    if isinstance(to_str(encryption), six.string_types):
+        encryption = xmltodict.parse(encryption)
+    BUCKET_ENCRYPTIONS[bucket_name] = encryption
+    return 200
+
+
+def set_object_lock(bucket_name, lock_config):
+    bucket_name = normalize_bucket_name(bucket_name)
+    # TODO: check if bucket exists, otherwise return 404-like error
+    if isinstance(to_str(lock_config), six.string_types):
+        lock_config = xmltodict.parse(lock_config)
+    OBJECT_LOCK_CONFIGS[bucket_name] = lock_config
+    return 200
+
+
+# -------------
+# UTIL METHODS
+# -------------
 
 
 def strip_chunk_signatures(data):
@@ -363,6 +492,8 @@ def bucket_exists(bucket_name):
     """Tests for the existence of the specified bucket. Returns the error code
     if the bucket does not exist (200 if the bucket does exist).
     """
+    bucket_name = normalize_bucket_name(bucket_name)
+
     s3_client = aws_stack.connect_to_service('s3')
     try:
         s3_client.head_bucket(Bucket=bucket_name)
@@ -381,14 +512,14 @@ def check_content_md5(data, headers):
     except Exception:
         expected = '__invalid__'
     if actual != expected:
-        result = {
-            'Error': {
-                'Code': 'InvalidDigest',
-                'Message': 'The Content-MD5 you specified was invalid'
-            }
-        }
-        content = xmltodict.unparse(result)
-        return requests_response(content, status_code=400)
+        return error_response('The Content-MD5 you specified was invalid', 'InvalidDigest', status_code=400)
+
+
+def error_response(message, code, status_code=400):
+    result = {'Error': {'Code': code, 'Message': message}}
+    content = xmltodict.unparse(result)
+    headers = {'content-type': 'application/xml'}
+    return requests_response(content, status_code=status_code, headers=headers)
 
 
 def expand_redirect_url(starting_url, key, bucket):
@@ -404,6 +535,27 @@ def expand_redirect_url(starting_url, key, bucket):
     return redirect_url
 
 
+def is_bucket_specified_in_domain_name(path, headers):
+    host = headers.get('host', '')
+    return re.match(r'.*s3(\-website)?\.([^\.]+\.)?amazonaws.com', host)
+
+
+def is_object_specific_request(path, headers):
+    """ Return whether the given request is specific to a certain S3 object.
+        Note: the bucket name is usually specified as a path parameter,
+        but may also be part of the domain name! """
+    bucket_in_domain = is_bucket_specified_in_domain_name(path, headers)
+    parts = len(path.split('/'))
+    return parts > (1 if bucket_in_domain else 2)
+
+
+def normalize_bucket_name(bucket_name):
+    bucket_name = bucket_name or ''
+    # AWS appears to automatically convert upper to lower case chars in bucket names
+    bucket_name = bucket_name.lower()
+    return bucket_name
+
+
 def get_bucket_name(path, headers):
     parsed = urlparse.urlparse(path)
 
@@ -414,7 +566,7 @@ def get_bucket_name(path, headers):
 
     # is the hostname not starting a bucket name?
     if host.startswith(HOSTNAME) or host.startswith(HOSTNAME_EXTERNAL):
-        return bucket_name
+        return normalize_bucket_name(bucket_name)
 
     # matches the common endpoints like
     #     - '<bucket_name>.s3.<region>.amazonaws.com'
@@ -441,7 +593,7 @@ def get_bucket_name(path, headers):
 
     # we're either returning the original bucket_name,
     # or a pattern matched the host and we're returning that name instead
-    return bucket_name
+    return normalize_bucket_name(bucket_name)
 
 
 def handle_notification_request(bucket, method, data):
@@ -502,6 +654,9 @@ class ProxyListenerS3(ProxyListener):
 
     def forward_request(self, method, path, data, headers):
 
+        # parse path and query params
+        parsed_path = urlparse.urlparse(path)
+
         # Make sure we use 'localhost' as forward host, to ensure moto uses path style addressing.
         # Note that all S3 clients using LocalStack need to enable path style addressing.
         if 's3.amazonaws.com' not in headers.get('host', ''):
@@ -515,6 +670,15 @@ class ProxyListenerS3(ProxyListener):
 
         modified_data = None
 
+        # check bucket name
+        bucket_name = get_bucket_name(path, headers)
+        if method == 'PUT' and not re.match(BUCKET_NAME_REGEX, bucket_name):
+            if len(parsed_path.path) <= 1:
+                return error_response('Unable to extract valid bucket name. Please ensure that your AWS SDK is ' +
+                    'configured to use path style addressing, or send a valid <Bucket>.s3.amazonaws.com "Host" header',
+                    'InvalidBucketName', status_code=400)
+            return error_response('The specified bucket is not valid.', 'InvalidBucketName', status_code=400)
+
         # TODO: For some reason, moto doesn't allow us to put a location constraint on us-east-1
         to_find = to_bytes('<LocationConstraint>us-east-1</LocationConstraint>')
         if data and data.startswith(to_bytes('<')) and to_find in data:
@@ -545,11 +709,14 @@ class ProxyListenerS3(ProxyListener):
         persistence.record('s3', method, path, data, headers)
 
         # parse query params
-        parsed = urlparse.urlparse(path)
-        query = parsed.query
-        path = parsed.path
+        query = parsed_path.query
+        path = parsed_path.path
         bucket = path.split('/')[1]
         query_map = urlparse.parse_qs(query, keep_blank_values=True)
+
+        # remap metadata query params (not supported in moto) to request headers
+        append_metadata_headers(method, query_map, headers)
+
         if query == 'notification' or 'notification' in query_map:
             # handle and return response for ?notification request
             response = handle_notification_request(bucket, method, data)
@@ -572,15 +739,55 @@ class ProxyListenerS3(ProxyListener):
         if query == 'replication' or 'replication' in query_map:
             if method == 'GET':
                 return get_replication(bucket)
+            if method == 'PUT':
+                return set_replication(bucket, data)
 
         if query == 'encryption' or 'encryption' in query_map:
             if method == 'GET':
                 return get_encryption(bucket)
+            if method == 'PUT':
+                return set_encryption(bucket, data)
+
+        if query == 'object-lock' or 'object-lock' in query_map:
+            if method == 'GET':
+                return get_object_lock(bucket)
+            if method == 'PUT':
+                return set_object_lock(bucket, data)
 
         if modified_data is not None:
             return Request(data=modified_data, headers=headers, method=method)
         return True
 
+    def get_201_reponse(self, key, bucket_name):
+        return """
+            <PostResponse>
+                <Location>{protocol}://{host}/{encoded_key}</Location>
+                <Bucket>{bucket}</Bucket>
+                <Key>{key}</Key>
+                <ETag>{etag}</ETag>
+            </PostResponse>
+            """.format(
+            protocol=get_service_protocol(),
+            host=config.HOSTNAME_EXTERNAL,
+            encoded_key=urlparse.quote(key, safe=''),
+            key=key,
+            bucket=bucket_name,
+            etag='d41d8cd98f00b204e9800998ecf8427f',
+        )
+
+    def get_forward_url(self, method, path, data, headers):
+        def sub(match):
+            # make sure to convert any bucket names to lower case
+            bucket_name = normalize_bucket_name(match.group(1))
+            return '/%s%s' % (bucket_name, match.group(2) or '')
+
+        path_new = re.sub(r'/([^?/]+)([?/].*)?', sub, path)
+        if path == path_new:
+            return
+        url = '%s://%s:%s%s' % (get_service_protocol(), constants.LOCALHOST,
+            constants.DEFAULT_PORT_S3_BACKEND, path_new)
+        return url
+
     def return_response(self, method, path, data, headers, response):
 
         path = to_str(path)
@@ -592,17 +799,25 @@ class ProxyListenerS3(ProxyListener):
         if (not bucket_name or len(bucket_name) == 0) and len(hostname_parts) > 1:
             bucket_name = hostname_parts[0]
 
-        # POST requests to S3 may include a success_action_redirect field,
-        # which should be used to redirect a client to a new location.
+        # POST requests to S3 may include a success_action_redirect or
+        # success_action_status field, which should be used to redirect a
+        # client to a new location.
         key = None
         if method == 'POST':
-            key, redirect_url = multipart_content.find_multipart_redirect_url(data, headers)
+            key, redirect_url = multipart_content.find_multipart_key_value(data, headers)
 
             if key and redirect_url:
                 response.status_code = 303
                 response.headers['Location'] = expand_redirect_url(redirect_url, key, bucket_name)
                 LOGGER.debug('S3 POST {} to {}'.format(response.status_code, response.headers['Location']))
 
+            key, status_code = multipart_content.find_multipart_key_value(
+                data, headers, 'success_action_status')
+            if response.status_code == 200 and status_code == '201' and key:
+                response.status_code = 201
+                response._content = self.get_201_reponse(key, bucket_name)
+                return response
+
         parsed = urlparse.urlparse(path)
         bucket_name_in_host = headers['host'].startswith(bucket_name)
 
@@ -641,16 +856,39 @@ class ProxyListenerS3(ProxyListener):
             response.status_code = 204
             return response
 
+        # emulate ErrorDocument functionality if a website is configured
+
+        if method == 'GET' and response.status_code == 404 and parsed.query != 'website':
+            s3_client = aws_stack.connect_to_service('s3')
+
+            try:
+                # Verify the bucket exists in the first place--if not, we want normal processing of the 404
+                s3_client.head_bucket(Bucket=bucket_name)
+                website_config = s3_client.get_bucket_website(Bucket=bucket_name)
+                error_doc_key = website_config.get('ErrorDocument', {}).get('Key')
+
+                if error_doc_key:
+                    error_object = s3_client.get_object(Bucket=bucket_name, Key=error_doc_key)
+                    response.status_code = 200
+                    response._content = error_object['Body'].read()
+                    response.headers['content-length'] = len(response._content)
+            except ClientError:
+                # Pass on the 404 as usual
+                pass
+
         if response:
             reset_content_length = False
 
-            # append CORS headers to response
+            # append CORS headers and other annotations/patches to response
             append_cors_headers(bucket_name, request_method=method, request_headers=headers, response=response)
             append_last_modified_headers(response=response)
+            append_list_objects_marker(method, path, data, response)
+            fix_location_constraint(response)
 
             # Remove body from PUT response on presigned URL
             # https://github.com/localstack/localstack/issues/1317
-            if method == 'PUT' and ('X-Amz-Security-Token=' in path or 'AWSAccessKeyId=' in path):
+            if method == 'PUT' and ('X-Amz-Security-Token=' in path or
+                    'X-Amz-Credential=' in path or 'AWSAccessKeyId=' in path):
                 response._content = ''
                 reset_content_length = True
 
@@ -668,17 +906,19 @@ class ProxyListenerS3(ProxyListener):
                     if param_name in query_map:
                         response.headers[header_name] = query_map[param_name][0]
 
-            # We need to un-pretty-print the XML, otherwise we run into this issue with Spark:
-            # https://github.com/jserver/mock-s3/pull/9/files
-            # https://github.com/localstack/localstack/issues/183
-            # Note: yet, we need to make sure we have a newline after the first line: <?xml ...>\n
             if response_content_str and response_content_str.startswith('<'):
                 is_bytes = isinstance(response._content, six.binary_type)
+                response._content = response_content_str
 
                 append_last_modified_headers(response=response, content=response_content_str)
 
-                # un-pretty-print the XML
-                response._content = re.sub(r'([^\?])>\n\s*<', r'\1><', response_content_str, flags=re.MULTILINE)
+                # We need to un-pretty-print the XML, otherwise we run into this issue with Spark:
+                # https://github.com/jserver/mock-s3/pull/9/files
+                # https://github.com/localstack/localstack/issues/183
+                # Note: yet, we need to make sure we have a newline after the first line: <?xml ...>\n
+                # Note: make sure to return XML docs verbatim: https://github.com/localstack/localstack/issues/1037
+                if method != 'GET' or not is_object_specific_request(path, headers):
+                    response._content = re.sub(r'([^\?])>\n\s*<', r'\1><', response_content_str, flags=re.MULTILINE)
 
                 # update Location information in response payload
                 response._content = self._update_location(response._content, bucket_name)
@@ -704,6 +944,8 @@ class ProxyListenerS3(ProxyListener):
                 response.headers['content-length'] = len(response._content)
 
     def _update_location(self, content, bucket_name):
+        bucket_name = normalize_bucket_name(bucket_name)
+
         host = config.HOSTNAME_EXTERNAL
         if ':' not in host:
             host = '%s:%s' % (host, config.PORT_S3)
@@ -717,8 +959,9 @@ class ProxyListenerS3(ProxyListener):
         if not query:
             return True
         # Except we do want to notify on multipart and presigned url upload completion
-        elif (method == 'POST' and query.startswith('uploadId')) or \
-                ('X-Amz-Credential' in query and 'X-Amz-Signature' in query):
+        contains_cred = 'X-Amz-Credential' in query and 'X-Amz-Signature' in query
+        contains_key = 'AWSAccessKeyId' in query and 'Signature' in query
+        if (method == 'POST' and query.startswith('uploadId')) or contains_cred or contains_key:
             return True
 
 
